'''
IzzyLib by Zoey Mae
Licensed under the CNPL: https://git.pixie.town/thufie/CNPL
https://git.barkshark.xyz/izaliamae/izzylib
'''

import os, sys, traceback
assert sys.version_info >= (3, 7)

__version_tpl__ = (0, 7, 5)
__version__ = '.'.join([str(v) for v in __version_tpl__])

from . import logging

izzylog = logging.logger['IzzyLib']
izzylog.set_config('level', os.environ.get('IZZYLOG_LEVEL', 'INFO'))

from .path import *
from .datestring import *
from .dotdict import *
from .misc import *
from .cache import *
from .color import Color
from .config import *
from .connection import *
from .url import *
from .object_base import ObjectBase
from .mastodon import MastodonApi
from .activitypub import *

from .http_client import (
	HttpClient,
	HttpClientRequest,
	HttpClientResponse
)

try:
	from .http_server import (
		Application,
		Request,
		Response
	)

except ImportError as e:
	izzylog.debug(f'Failed to import http_server: {e}')


def register_global(obj, name=None):
	# This doesn't work and I'm not sure why
	#assert isinstance(obj, (callable, object))

	__builtins__[name or class_name(obj)] = obj

	return obj


def add_builtins(*classes, **kwargs):
	new_builtins = [
		BaseConfig,
		DateString,
		DotDict,
		HttpClient,
		JsonConfig,
		LruCache,
		ObjectBase,
		Path,
		TtlCache,
		Url,
		convert_to_boolean,
		convert_to_bytes,
		convert_to_string,
		logging,
		random_gen,
		random_int,
		random_str,
		*classes
	]

	for cls in new_builtins:
		register_global(cls)

	for name, cls in kwargs.items():
		register_global(cls, name)
