import enum


__all__ = []


def register(enum):
	__all__.append(enum.__name__)
	setattr(enum, 'from_index', classmethod(enum_from_index))
	setattr(enum, 'index', classmethod(enum_index))
	setattr(enum, 'parse', classmethod(enum_parse))
	return enum


def enum_from_index(cls, key):
	for idx, value in enumerate(cls):
		if idx == key:
			return value

	raise IndexError(key)


def enum_index(cls, item):
	item = cls.parse(item)

	for idx, value in enumerate(cls):
		if value == item:
			return idx

	raise ValueError(item)


def enum_parse(cls, key):
	if isinstance(key, cls):
		return key

	try: return cls[key]
	except KeyError: pass

	try: return cls(key)
	except ValueError: pass

	for item in cls:
		if isinstance(item.value, str) and key.lower() == item.value.lower():
			return item

		elif key.lower() == item.name.lower():
			return item



@register
class Enum(enum.Enum):
	pass


@register
class StrEnum(str, enum.Enum):
	def __str__(self):
		return self.value


@register
class IntEnum(enum.IntEnum):
	pass


@register
class IntFlagEnum(enum.IntFlag):
	pass


@register
class FlagEnum(enum.Flag):
	pass

