## TODO: use izzylib.enum.Enum instead of built-in enum class

from .enum import StrEnum, IntEnum


__all__ = []


def register(enum):
	__all__.append(enum.__name__)
	return enum


@register
class DatetimeFormat(StrEnum):
	HTTP = '%a, %d %b %Y %H:%M:%S GMT'
	ACTIVITYPUB = '%Y-%m-%dT%H:%M:%SZ'
	ACTIVITYPUB_MICRO = '%Y-%m-%dT%H:%M:%S.%fZ'
	LINUX_TRASH = '%Y-%m-%dT%H:%M:%S'
	MASTODON = '%Y-%m-%d'


@register
class FileSizeUnit(StrEnum):
	BYTE = 'B'

	KIBIBYTE = 'KiB'
	MEBIBYTE = 'MiB'
	GIBIBYTE = 'GiB'
	TEBIBYTE = 'TiB'
	PEBIBYTE = 'PiB'
	EXBIBYTE = 'EiB'
	ZEBIBYTE = 'ZiB'
	YOBIBYTE = 'YiB'

	KILOBYTE  = 'KB'
	MEGABYTE  = 'MB'
	GIGABYTE  = 'GB'
	TERABYTE  = 'TB'
	PETABYTE  = 'PB'
	EXABYTE   = 'EB'
	ZETTABYTE = 'ZB'
	YOTTABYTE = 'YB'

	B = BYTE
	K = KIBIBYTE
	M = MEBIBYTE
	G = GIBIBYTE
	T = TEBIBYTE
	P = PEBIBYTE
	E = EXBIBYTE
	Z = ZEBIBYTE
	Y = YOBIBYTE


@register
class FileSizeMulti(IntEnum):
	BYTE = 1

	KIBIBYTE = 1024
	MEBIBYTE = 1024 ** 2
	GIBIBYTE = 1024 ** 3
	TEBIBYTE = 1024 ** 4
	PEBIBYTE = 1024 ** 5
	EXBIBYTE = 1024 ** 6
	ZEBIBYTE = 1024 ** 7
	YOBIBYTE = 1024 ** 8

	KILOBYTE  = 1000
	MEGABYTE  = 1000 ** 2
	GIGABYTE  = 1000 ** 3
	TERABYTE  = 1000 ** 4
	PETABYTE  = 1000 ** 5
	EXABYTE   = 1000 ** 6
	ZETTABYTE = 1000 ** 7
	YOTTABYTE = 1000 ** 8

	B = BYTE
	K = KIBIBYTE
	M = MEBIBYTE
	G = GIBIBYTE
	T = TEBIBYTE
	P = PEBIBYTE
	E = EXBIBYTE
	Z = ZEBIBYTE
	Y = YOBIBYTE


@register
class HttpMethod(StrEnum):
	CONNECT = 'CONNECT'
	DELETE = 'DELETE'
	GET = 'GET'
	HEAD = 'HEAD'
	OPTIONS = 'OPTIONS'
	PATCH = 'PATCH'
	POST = 'POST'
	PUT = 'PUT'
	TRACE = 'TRACE'


@register
class IpAddressType(StrEnum):
	PUBLIC = 'public'
	PRIVATE = 'private'
	LOOPBACK = 'loopback'
	LOCAL = 'local'
	MULTICAST = 'multicast'
	RESERVED = 'reserved'
	UNKNOWN = 'unknown'


@register
class IpAddressVersion(StrEnum):
	IPV4 = 'IPv4'
	IPV6 = 'IPv6'


@register
class LogLevel(IntEnum):
	CRITICAL = 60
	ERROR = 50
	WARNING = 40
	INFO = 30
	VERBOSE = 20
	DEBUG = 10
	MERP = 0


@register
class PathType(StrEnum):
	DIR = 'dir'
	FILE = 'file'
	LINK = 'link'
	UNKNOWN = 'unknown'


@register
class Protocol(IntEnum):
	FTP = 21
	SSH = 22
	SMTP = 25
	DNS = 53
	TFTP = 69
	HTTP = 80
	WS = 80
	HTTPS = 443
	WSS = 443
	FTPS = 990
	MYSQL = 3306
	PSQL = 5432
