from ..datestring import DateString
from ..dotdict import DotDict
from ..enums import DatetimeFormat
from ..object_base import ObjectBase
from ..path import Path
from ..url import Address


class Request(ObjectBase):
	def __init__(self, handler):
		super().__init__(
			address = Address(handler.client_address[0]),
			app = handler.app,
			ctx = DotDict(),
			method = handler.method,
			path = None,
			query = None
		)

		self.__handler = handler

		try:
			path, query = handler.path.split('?', 1)

		except:
			path, query = handler.path, ''

		self.path = Path(path)
		self.query = DotDict.new_from_query_string(query)

		self.__readonly = True


	def __getattr__(self, key):
		try:
			return object.__getattribute__(self, 'ctx')[key]

		except (KeyError, AttributeError):
			pass

		return super().__getattr__(key)


	def __getitem__(self, key):
		return self.ctx[key]


	def __setitem__(self, key, value):
		self.ctx[key] = value


	def __delitem__(self, key):
		del self.ctx[key]


	@property
	def agent(self):
		return self.headers.get('User-Agent', 'no agent')


	@property
	def accept(self):
		return self.headers.get('Accept', '')


	@property
	def content_type(self):
		return self.headers.get('Content-Type', '')


	@property
	def date(self):
		date_str = self.headers.get('Date')

		if date_str:
			return DateString(date_str, DatetimeFormat.HTTP)


	@property
	def headers(self):
		return self.__handler.headers


	@property
	def host(self):
		return self.headers.get('Host', self.app.host)


	@property
	def length(self):
		return int(self.headers.get('Content-Length', 0))


	@property
	def protocol(self):
		return self.headers.get('X-Forwarded-Proto', 'http')


	@property
	def remote(self):
		try:
			return Address(self.headers.get('X-Real-Ip', self.headers.get('X-Forwarded-For', self.address)))

		except:
			return None


	def body(self):
		return self.__handler.read(self.length)
