import json

from ..dotdict import DotDict
from ..misc import convert_to_bytes


class Response:
	def __init__(self, body=b'', status=200, headers={}, content_type='text/plain'):
		self._body = body

		self.status = status
		self.headers = DotDict()
		self.headers['Content-Type'] = content_type


	@classmethod
	def new_text(cls, *args, **kwargs):
		return cls(*args, **kwargs)


	@classmethod
	def new_html(cls, *args, **kwargs):
		response = cls()
		response.set_html(*args, **kwargs)

		return response


	@classmethod
	def new_css(cls, *args, **kwargs):
		response = cls()
		response.set_css(*args, **kwargs)

		return response


	@classmethod
	def new_javascript(cls, *args, **kwargs):
		response = cls()
		response.set_javascript(*args, **kwargs)

		return response


	@classmethod
	def new_json(cls, *args, **kwargs):
		response = cls()
		response.set_json(*args, **kwargs)

		return response


	@property
	def body(self):
		return self._body


	@body.setter
	def body(self, value):
		self._body = convert_to_bytes(value)
		self.headers['Content-Length'] = len(self._body)


	def set_data(self, body=None, status=None, headers={}, content_type=None):
		if body:
			self.body = body

		if status:
			self.status = status

		self.headers.update(headers)

		if content_type:
			self.headers['Content-Type'] = content_type


	def set_html(self, body=b'', status=200, headers={}):
		self.set_data(body, status, headers, 'text/html')


	def set_css(self, body=b'', headers={}):
		self.set_data(body, headers=headers, content_type='text/css')


	def set_javascript(self, body=b'', headers={}):
		self.set_data(body, headers=headers, content_type='application/javascript')


	def set_json(self, body={}, status=200, headers={}, activity=False, indent=None):
		assert isinstance(body, dict), 'Body must be a dict'

		if isinstance(body, DotDict):
			data = body.to_json(indent)

		else:
			data = json.dumps(body, indent=indent)

		content_type = 'application/activity+json' if activity else 'application/json'
		self.set_data(data, status, headers, content_type)
