import re

from datetime import datetime
from izzylib import DotDict, Path
from izzylib.template import Color

from . import VERSION, error, logging
from .base import FileHandler, HttpView, HttpMiddleware, cache_control, log_async_function_time
from .request import Request
from .response import Response


variable_types = {
	'float': float,
	'int': int,
	'str': str
}


## Copied from old asgi server and needs to be modified
class Router:
	def __init__(self, app):
		self.app = app
		self.cfg = app.cfg
		self.handlers = {}
		self.filehandlers = {}
		self.matchdata = {}


	def set_default_template_routes(self):
		self.add_static(Path(__file__).parent.join('frontend/static'), '/asgi/static')
		self.add_route(AsgiStyle, '/asgi/style.css')
		self.add_route(AsgiManifest, '/asgi/manifest.json')


	async def handle_request(self, request, response):
		keywords = {}

		## Is it a static file path?
		for basepath, data in self.filehandlers.items():
			if request.method.lower() == 'get' and request.path.startswith(basepath):
				handler = data[0](self.app, request, response)

				await handler.get(request, response, basepath, data[1])
				return handler.response

		try:
			## Is it a normal path?
			handler = self.handlers[request.path](self.app, request, response)

		except KeyError:
			## Is it a path with keywords?
			for path, match_data in self.matchdata.items():
				keywords = self.match_path(match_data, path, request.path)

				if keywords:
					break

			if not keywords:
				raise error.HttpError(f'Url not found: {request.path}', 404)

			handler = self.handlers[path](self.app, request, response)

		try:
			function = getattr(handler, request.method.lower())

		except AttributeError:
			raise error.HttpError('Method not supported for this url', 405)

		return await log_async_function_time(handler.__class__.__name__, function(request, response, **keywords))


	def get_route(self, method, path):
		return self.handlers[path]


	def add_route(self, handler_class, path):
		if not path.startswith('/'):
			path = '/' + path

		try:
			self.handlers[path]
			raise error.HandlerExistsError(f'Handler for "{path}" already exists')

		except KeyError:
			self.matchdata[path] = self.parse_path(path)
			self.handlers[path] = handler_class

			# This needs to be handled better
			if self.template:
				return
				menu_name = handler_class.name

				if menu_name:
					self.template.cfg.menu[menu_name] = path


	def add_static(self, basepath, webpath):
		if not str(webpath).startswith('/'):
			webpath = Path('/' + str(webpath))

		try:
			self.filehandlers[basepath]
			raise error.HandlerExistsError(f'File handler for "{webpath}" already exists')

		except KeyError:
			self.filehandlers[basepath] = (FileHandler, webpath)


	def parse_path(self, path):
		#path = '/@{actor:str}/status/{postid:int}uvu'
		parts = path.split('/')[1:]
		pattern = re.compile(r'^(.*)(\{.*:.*\})(.*)$')
		data = {}

		for idx, part in enumerate(parts):
			match = pattern.match(part)

			if match:
				key, vartype = match.group(2).strip('[]').split(':')

				if vartype not in ['int', 'str', '']:
					raise ValueError('Variable type is not "int", "string", or blank')

				line = {
					'key': key,
					'type': variable_types.get(vartype, str),
					'pre': match.group(1),
					'post': match.group(3),
					'path': path
				}

				if any(map(pattern.match, [line['pre'], line['post']])):
					raise ValueError('Cannot use more than 1 variable per path segment')

				data[idx] = (line)

		return data


	def match_path(self, path_data, method_path, path):
		if method_path == path:
			return True

		parts = path.split('/')[1:]
		mparts = method_path.split('/')[1:]
		variables = {}

		if len(parts) != len(mparts):
			return False

		for idx, part in enumerate(parts):
			try:
				if mparts[idx] == part:
					continue

			except IndexError:
				return False

			try:
				data = path_data[idx]
				pre = data['pre']
				post = data['post']

			except KeyError as e:
				return False

			if (pre or post) and not (part.startswith(pre) or part.endswith(post)):
				return False

			part = part.replace(pre, '', 1).replace(post, '', -1)
			variables[data['key']] = data['type'](part)

		return variables


class PathMatch(ObjectBase):
	def __init__()
						'key': key,
					'type': variable_types.get(vartype, str),
					'pre': match.group(1),
					'post': match.group(3),
					'path': path
