from .response import Response

from .. import exceptions
from .. import activitypub as ap
from ..http_utils import methods as http_methods
from ..url import Url


class View:
	app = None
	paths = []


	def __init__(self, app, *args, **kwargs):
		self.app = app
		self.args = args
		self.kwargs = kwargs


	def get_handler(self, method):
		if method.upper() not in http_methods:
			raise ValueError(f'Invalid HTTP method: {method}')

		return getattr(self, method.lower())



class WellKnownWebfinger(View):
	paths = ['/.well-known/webfinger']

	def __init__(self, app, actor_path, profile_path=None):
		super().__init__(app)

		self.actor_path = actor_path
		self.profile_path = profile_path


	## re-define this function and return a boolean
	def match_user(self, handle, domain):
		return False


	def get(self, request):
		resource = request.query.get('resource')

		if not resource:
			return Response('Missing resource', status=400)

		account = resource.replace('acct:', '')

		try:
			handle, domain = account.split('@', 1)

		except:
			raise exceptions.HttpBadRequest('Invalid account format')

		if not self.match_user(handle, domain):
			return Response.new_text('User does not exist', status=404)

		urls = [
			Url.new(request.app.host, self.actor_path.format(handle=handle))
		]

		if self.profile_path:
			urls.append(Url.new(request.app.host, self.profile_path.format(handle=handle)))

		webfinger = ap.Webfinger.new(handle, request.app.host, *urls)

		return Response.new_json(webfinger, indent=4)


def actor(request, handle):
	actor = f'https://{request.app.host}/actor/{handle}'
	pubkey = None

	return Response.new_json(
		ap.Actor.new(actor, handle, None),
		activity = True,
		indent = 4
	)
